#!/bin/sh
###############################################################################
# IBM Confidential
# OCO Source Materials
# © Copyright IBM Corporation 2018
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with
# the U.S. Copyright office.
###############################################################################
error() {
    cd ${SPE_INSTALL_DIR}
    # DO NOT REMOVE THE WORK_DIR FOR PROBLEM DETERMINATION
    echo
    echo $1
    echo
    exit 1
}

syntax() {
    echo "Deploys pack ${resourceType} resources"
    echo
    echo "$0 -j /path/to/patch.jar [ -t tenantID ] [ -u db|nodb ]"
    echo
    echo "Command line arguments:"
    echo "-j  (Required) Full path to the resource jar file"
    echo "-t  (Optional) Tenant ID under which the resources should be imported."
    echo "    It defaults to the 'DEFAULT' tenant if not specified."  
    echo "-u  (Optional) The 'db' value indicates that the common resources need to be imported."
    echo "    It defaults to the 'db' value if not specified. Specify 'nodb' to skip the import."
    echo
}

if [ -z "$InSPEEnv" ]; then
    echo Please initialize the SPE environment before running this command.
    echo You can use spesetup.sh to initialize the environment.
else
    TIMESTAMP="D"`date +%Y%m%d`".T"`date +%H%M%S`
    PACK_V_R_M=
    WIN_JAR=
    REF_ID=DEFAULT
    updateDB=TRUE
    updateDBArgument=db
    JAR_WTX=D3ADB33F
    resourceType=patch
  
    # Get input
    while getopts j:d:t:u:i option
    do
        case ${option} in
            j) WIN_JAR=${OPTARG}
               ;;
            t) REF_ID=${OPTARG}
               ;;
            u) updateDBArgument=${OPTARG}
               ;;
            *) syntax
               exit
               ;;
        esac
    done

    # Validate the WIN_JAR
    if [ -z "${WIN_JAR}" ]
    then
        echo "Missing mandatory parameter -j"
        syntax
        exit 1
    else
        if [ ! -f "${WIN_JAR}" ]
        then
            if [ "${WIN_JAR}" != "nojar" ]
            then
                error "Resource jar file does not exist: ${WIN_JAR}"
            fi
        else            
            if [ "${WIN_JAR##*.}" != "jar" ]
            then
                error "Invalid jar file name: ${WIN_JAR}"
            fi
        fi    
    fi

    # Validate the REF_ID
    if  [ "`echo ${REF_ID} | cut -c1`" = "-" ]
    then
        error "Please specify a valid tenantID after the '-t' option"
    fi

    # Validate the updateDBArgument
    if  [ "${updateDBArgument}" = "db" ]
    then
        updateDB=TRUE
    else
        if  [ "${updateDBArgument}" = "nodb" ]
        then
            updateDB=FALSE
        else
            error "Please specify 'db' OR 'nodb' after the '-u' option"
        fi
    fi

    echo WIN_JAR=${WIN_JAR}
    echo tenantID=${REF_ID}
    echo updateDB=${updateDB}
    echo

    if [ "${JAR_WTX}" != "nono" ]
    then
        # Generate WORK_DIR name and extract jar to it
        JAR_NAME=`basename "${WIN_JAR}"`
        echo JAR_NAME=${JAR_NAME}
        NO_JAR_NAME=${JAR_NAME%.jar}
        WORK_DIR="${SPE_INSTALL_DIR}/${NO_JAR_NAME}"
        echo WORK_DIR=${WORK_DIR}
        if [ -d ${WORK_DIR} ]
        then
            rm -rf ${WORK_DIR}
        fi

        echo
        echo "Extracting files..."
        ${JAVA_HOME}/bin/java com.ibm.spe.core.utils.ExtractJarFile -name="${WIN_JAR}" -output="${WORK_DIR}"
        if [ $? -ne 0 ]
        then
            error "Failed to extract ${WIN_JAR} to ${WORK_DIR}"
        fi    
        cd "${WORK_DIR}"
        if [ $? -ne 0 ]
        then
            error "Failed to change directory to ${WORK_DIR}"
        fi    

        echo
        echo Verifying the OS using the UNAME file
        targetOS=
        localOS=`uname -s`
        if [ -f UNAME ]
        then
            if [ ${localOS} = "OS/390" ]
            then
                targetOS=`cat UNAME | tr -d '\r' | iconv -f ISO8859-1 -t IBM-1047`
            else
                targetOS=`cat UNAME | tr -d '\r'`
            fi
            echo Target OS=[${targetOS}]
            echo Local OS=[${localOS}]
            echo ${targetOS} | grep ${localOS} > /dev/null
            if [ $? -ne 0 ]
            then
                error "Could not find a match for Local OS in Target OS"
            else
                echo Local OS matches the target one
            fi
        else
            error "Missing the UNAME file. Cannot determine the target platform"
        fi

        cd "${WORK_DIR}"
        PACK=
        if [ -f PACK ]
        then
            if [ ${localOS} = "OS/390" ]
            then
                PACK=`cat PACK | tr -d '\r' | iconv -f ISO8859-1 -t IBM-1047`
            else
                PACK=`cat PACK | tr -d '\r'`
            fi
        else
            error "Missing the PACK file. Cannot determine the pack type"
        fi

        PACK_VERSION=
        if [ -f PACK_VERSION ]
        then
            if [ ${localOS} = "OS/390" ]
            then
                PACK_VERSION=`cat PACK_VERSION | tr -d '\r' | iconv -f ISO8859-1 -t IBM-1047`
            else
                PACK_VERSION=`cat PACK_VERSION | tr -d '\r'`
            fi
            echo Bundle Jar Version=[${PACK_VERSION}]
            PACK_V_R_M=`echo ${PACK_VERSION} | cut -c1-5`
        else
            error "Missing the PACK_VERSION file. Cannot determine the ${PACK} target version"
        fi

        echo
        echo Verifying the pack version using the PACK_VERSION file
        PROP_VER=properties/version
        EDI_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Supply_Chain_EDI_Server-${PACK_V_R_M}.swidtag
        HC_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Healthcare_server-${PACK_V_R_M}.swidtag
        FSP_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Financial_Payments_server-${PACK_V_R_M}.swidtag

        if [ ${PACK} = "edi" ] && [ ! -f "${SPE_INSTALL_DIR}/${PROP_VER}/${EDI_SWTAG_FILE}" ] 
        then
            error "The ${PACK} ${PACK_V_R_M} pack is not installed, you cannot deploy this patch bundle"
        fi
        if [ ${PACK} = "hc" ] && [ ! -f "${SPE_INSTALL_DIR}/${PROP_VER}/${HC_SWTAG_FILE}" ] 
        then
            error "The ${PACK} ${PACK_V_R_M} pack is not installed, you cannot deploy this patch bundle"
        fi
        if [ ${PACK} = "fsp" ] && [ ! -f "${SPE_INSTALL_DIR}/${PROP_VER}/${FSP_SWTAG_FILE}" ] 
        then
            error "The ${PACK} ${PACK_V_R_M} pack is not installed, you cannot deploy this patch bundle"
        fi

        echo
        echo Verifying the resource type using the RESOURCE file
        resourceType=
        if [ -f RESOURCE ]
        then
            if [ ${localOS} = "OS/390" ]
            then
                resourceType=`cat RESOURCE | tr -d '\r' | iconv -f ISO8859-1 -t IBM-1047`
            else
                resourceType=`cat RESOURCE | tr -d '\r'`
            fi
            echo Resource bundle=[${resourceType}]
            echo Expected bundle=[patch]
        else
            error "Missing the RESOURCE file. Cannot determine the resource type"
        fi

        if [ "${resourceType}" != "patch" ]
        then
            error "Invalid resource type ${resourceType}"
        fi

        cd ${WORK_DIR}
        echo
        echo "Deploying ${PACK} ${PACK_V_R_M} ${resourceType}"
        echo
        echo "Removing accessory files"
        rm -rf PACK
        rm -rf PACK_VERSION
        rm -rf RESOURCE
        rm -rf UNAME
        rm -rf META-INF

        echo
        BACKUP_DIR=backups/PRE-${NO_JAR_NAME}-PATCH-${TIMESTAMP}
        echo Creating ${BACKUP_DIR} directories...
        #Get list of directories
        LIST_OF_DIRS=${NO_JAR_NAME}.dirs
        find * -type d > ../${LIST_OF_DIRS}
        for FILE in $(cat ../${LIST_OF_DIRS})
        do
            TARGET_DIR="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
            if [ ! -d "${TARGET_DIR}" ]
            then
                mkdir -p "${TARGET_DIR}"
                if [ $? -ne 0 ]
                then
                    error "Failed to create ${TARGET_DIR} directory"
                fi
            fi
        done

        # Get list of files to copy
        LIST_OF_FILES=${NO_JAR_NAME}.txt
        find * -type f > ../${LIST_OF_FILES}

        echo
        echo Deploying files...
        # Copy binary files
        LIST_OF_BINARY=${NO_JAR_NAME}.binary
        egrep "\.jar|\.mmc|edisvu\.|hcsvu\.|\.txo|\.noconv" ../${LIST_OF_FILES} > ../${LIST_OF_BINARY}
        for FILE in $(cat ../${LIST_OF_BINARY})
        do
            TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
            BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
            if [ -f ${TARGET_FILE} ]
            then
                echo "backing up old ${FILE}"
                mv ${TARGET_FILE} ${BACKUP_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to make a backup copy of ${FILE}"
                fi
                echo "deploying ${FILE}"
                cp -p ${FILE} ${TARGET_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to deploy ${FILE}"
                fi
            else
                echo "skipping ${FILE}"
            fi
        done

        # iconv (z/OS only) and copy conv files
        LIST_OF_CONV=${NO_JAR_NAME}.conv
        egrep "\.conv" ../${LIST_OF_FILES} > ../${LIST_OF_CONV}
        for FILE in $(cat ../${LIST_OF_CONV})
        do
            TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
            BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
            if [ -f ${TARGET_FILE} ]
            then
                echo "backing up old ${FILE}"
                mv ${TARGET_FILE} ${BACKUP_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to make a backup copy of ${FILE}"
                fi
                echo "deploying ${FILE}"
                if [ ${localOS} = "OS/390" ]
                then
                    iconv -f ISO8859-1 -t IBM-1047 ${FILE} > ${TARGET_FILE}
                else
                    cp -p ${FILE} ${TARGET_FILE}
                fi
                if [ $? -ne 0 ]
                then
                    error "Failed to deploy ${FILE}"
                fi
            else
                echo "skipping ${FILE}"
            fi
        done

        # Fix line terminator, iconv (z/OS only) and copy text files
        LIST_OF_TEXT=${NO_JAR_NAME}.text
        egrep "\.txt|\.dat|\.ini|\.prop|\.sh" ../${LIST_OF_FILES} > ../${LIST_OF_TEXT}
        for FILE in $(cat ../${LIST_OF_TEXT})
        do
            TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
            BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
            if [ -f ${TARGET_FILE} ]
            then
                echo "backing up old ${FILE}"
                mv ${TARGET_FILE} ${BACKUP_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to make a backup copy of ${FILE}"
                fi
                echo "deploying ${FILE}"
                if [ ${localOS} = "OS/390" ]
                then
                    LC_CTYPE=C tr -d '\r' < ${FILE}  | iconv -f ISO8859-1 -t IBM-1047 > ${TARGET_FILE}
                else
                    LC_CTYPE=C tr -d '\r' < ${FILE} > ${TARGET_FILE}
                fi
                if [ $? -ne 0 ]
                then
                    error "Failed to deploy ${FILE}"
                fi
            else
                echo "skipping ${FILE}"
            fi
        done

        # Fix line terminator and copy xml files
        LIST_OF_XML=${NO_JAR_NAME}.xml
        egrep -v "\.jar|\.mmc|edisvu\.|hcsvu\.|\.txo|\.noconv|\.conv|\.txt|\.dat|\.ini|\.prop|\.sh" ../${LIST_OF_FILES} > ../${LIST_OF_XML}
        for FILE in $(cat ../${LIST_OF_XML})
        do
            TARGET_FILE="${SPE_INSTALL_DIR}/${FILE}"
            BACKUP_FILE="${SPE_INSTALL_DIR}/${BACKUP_DIR}/${FILE}"
            if [ -f ${TARGET_FILE} ]
            then
                echo "backing up old ${FILE}"
                mv ${TARGET_FILE} ${BACKUP_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to make a backup copy of ${FILE}"
                fi
                echo "deploying ${FILE}"
                tr -d '\r' < ${FILE} > ${TARGET_FILE}
                if [ $? -ne 0 ]
                then
                    error "Failed to deploy ${FILE}"
                fi
            else
                echo "skipping ${FILE}"
            fi
        done

        cd ${SPE_INSTALL_DIR}
        # Cleanup
        rm -rf ${LIST_OF_DIRS}
        rm -rf ${LIST_OF_BINARY}
        rm -rf ${LIST_OF_CONV}
        rm -rf ${LIST_OF_TEXT}
        rm -rf ${LIST_OF_XML}
        rm -rf ${LIST_OF_FILES}
    fi
   
    DBPROPS="dbprops=${SPE_INSTALL_DIR}/dbprops.cfg"

    # Import pack resources
    cd ${WORK_DIR}
    
    if [ "${PACK}" = "edi" ]
    then 
        echo
        if [ "${updateDB}" = "TRUE" ]
        then
            if [ -f "${SPE_INSTALL_DIR}/schema/SPE_IE_Standard_Types_X12.xsd" ]
            then        
                ${JAVA_HOME}/bin/java com.ibm.spe.core.utils.SPEImporter ${DBPROPS} quiet updateExisting updateDefaultVersion -xsd="${SPE_INSTALL_DIR}/schema/SPE_IE_Standard_Types_X12.xsd" refId=${REF_ID}
                if [ $? -ne 0 ]
                then
                    error "There was an error while loading ${SPE_INSTALL_DIR}/schema/SPE_IE_Standard_Types_X12.xsd"
                fi
            fi
        fi
    fi 

    if [ "${PACK}" = "hc" ]
    then
        echo
    fi

    if [ "${PACK}" = "fsp" ]
    then
        echo 
    fi

    cd ${SPE_INSTALL_DIR}
    echo
    rm -rf ${WORK_DIR}
    echo "${PACK} ${PACK_V_R_M} patch files were deployed, configured, and loaded successfully"
    echo
fi
